// src/index.ts
import sanitizeHtml from "sanitize-html";
var RegShakuDirectiveUnderlineSolid = /^(?<leadingSpaces>\s*)(?<content>(-|_)+)\s*(?<shift><*)\s*$/;
var RegShakuDirectiveUnderlineWavy = /^(?<leadingSpaces>\s*)(?<content>~+)\s*(?<shift><*)\s*$/;
var RegShakuDirectiveUnderlineDotted = /^(?<leadingSpaces>\s*)(?<content>\.+)\s*(?<shift><*)\s*$/;
var RegShakuAnnotationLine = /^(?<leadingSpaces>\s*)\[(?<content>.+)\]\s*(?<shift><*)\s*$/;
var RegShakuDirectiveCallout = /^(?<leadingSpaces>\s*)\^\s*(?<shift><*)\s*$/;
var RegShakuDirectiveCollapse = /^(?<leadingSpaces>\s*)@collapse\s+(?<mark>\S+)\s*$/;
var RegShakuDirectiveHighlight = /^(?<leadingSpaces>\s*)@highlight(\s+(?<mark>(\S+)?)\s*)?$/;
var RegShakuDirectiveDim = /^(?<leadingSpaces>\s*)@dim(\s+(?<mark>(\S+)?)\s*)?$/;
var RegShakuDirectiveFocus = /^(?<leadingSpaces>\s*)@focus(\s+(?<mark>(\S+)?)\s*)?$/;
var parseLine = (line) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j, _k, _l, _m, _n, _o, _p, _q, _r, _s;
  const matchShakuDirectiveUnderlineSolid = line.match(
    RegShakuDirectiveUnderlineSolid
  );
  if (matchShakuDirectiveUnderlineSolid) {
    return {
      type: "DirectiveUnderline",
      config: {
        offset: (((_a = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _a.leadingSpaces.length) ?? 0) - (((_b = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _b.shift.length) ?? 0),
        content: ((_c = matchShakuDirectiveUnderlineSolid.groups) == null ? void 0 : _c.content) ?? "",
        style: "solid"
      }
    };
  }
  const matchShakuDirectiveUnderlineWavy = line.match(
    RegShakuDirectiveUnderlineWavy
  );
  if (matchShakuDirectiveUnderlineWavy) {
    return {
      type: "DirectiveUnderline",
      config: {
        offset: (((_d = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _d.leadingSpaces.length) ?? 0) - (((_e = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _e.shift.length) ?? 0),
        content: ((_f = matchShakuDirectiveUnderlineWavy.groups) == null ? void 0 : _f.content) ?? "",
        style: "wavy"
      }
    };
  }
  const matchShakuDirectiveUnderlineDotted = line.match(
    RegShakuDirectiveUnderlineDotted
  );
  if (matchShakuDirectiveUnderlineDotted) {
    return {
      type: "DirectiveUnderline",
      config: {
        offset: (((_g = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _g.leadingSpaces.length) ?? 0) - (((_h = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _h.shift.length) ?? 0),
        content: ((_i = matchShakuDirectiveUnderlineDotted.groups) == null ? void 0 : _i.content) ?? "",
        style: "dotted"
      }
    };
  }
  const matchShakuAnnotationLine = line.match(RegShakuAnnotationLine);
  if (matchShakuAnnotationLine) {
    return {
      type: "AnnotationLine",
      config: {
        offset: (((_j = matchShakuAnnotationLine.groups) == null ? void 0 : _j.leadingSpaces.length) ?? 0) - (((_k = matchShakuAnnotationLine.groups) == null ? void 0 : _k.shift.length) ?? 0),
        content: ((_l = matchShakuAnnotationLine.groups) == null ? void 0 : _l.content) ?? ""
      }
    };
  }
  const matchShakuDirectiveCallout = line.match(RegShakuDirectiveCallout);
  if (matchShakuDirectiveCallout) {
    return {
      type: "DirectiveCallout",
      config: {
        offset: (((_m = matchShakuDirectiveCallout.groups) == null ? void 0 : _m.leadingSpaces.length) ?? 0) - (((_n = matchShakuDirectiveCallout.groups) == null ? void 0 : _n.shift.length) ?? 0)
      }
    };
  }
  const matchShakuDirectiveCollapse = line.match(RegShakuDirectiveCollapse);
  if (matchShakuDirectiveCollapse) {
    const mark = (_o = matchShakuDirectiveCollapse.groups) == null ? void 0 : _o.mark;
    if (mark === "start" || mark === "end") {
      return {
        type: "DirectiveCollapse",
        config: {
          offset: ((_p = matchShakuDirectiveCollapse.groups) == null ? void 0 : _p.leadingSpaces.length) ?? 0,
          mark
        }
      };
    }
  }
  const matchShakuDirectiveHighlight = line.match(RegShakuDirectiveHighlight);
  if (matchShakuDirectiveHighlight) {
    const mark = ((_q = matchShakuDirectiveHighlight.groups) == null ? void 0 : _q.mark) || "below";
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveHighlight",
        config: {
          mark
        }
      };
    }
  }
  const matchShakuDirectiveDim = line.match(RegShakuDirectiveDim);
  if (matchShakuDirectiveDim) {
    const mark = ((_r = matchShakuDirectiveDim.groups) == null ? void 0 : _r.mark) || "below";
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveDim",
        config: {
          mark
        }
      };
    }
  }
  const matchShakuDirectiveFocus = line.match(RegShakuDirectiveFocus);
  if (matchShakuDirectiveFocus) {
    const mark = ((_s = matchShakuDirectiveFocus.groups) == null ? void 0 : _s.mark) || "below";
    if (mark === "start" || mark === "end" || mark === "below") {
      return {
        type: "DirectiveFocus",
        config: {
          mark
        }
      };
    }
  }
  return null;
};
function shouldApplyAnnotation(meta) {
  return /\bannotate\b/.test(meta);
}
function renderComponent(component) {
  switch (component.type) {
    case "ShakuComponentCallout":
      return `<div class="shaku-callout" style="left:${component.config.offset}ch"><span class="shaku-callout-arrow" style="left:${component.config.arrowOffset}ch"></span>${sanitize(component.config.contents)}</div>`;
    case "ShakuComponentUnderline":
      return `<div class="shaku-underline shaku-underline-${component.config.underlineStyle}" style="left:${component.config.offset}ch"><span class="shaku-underline-line" style="left:${component.config.underlineOffset}ch">${component.config.underlineContent}</span>${sanitize(
        component.config.contents
      )}</div>`;
    default:
      assertsNever(component);
  }
}
function assertsNever(data) {
  throw new Error("expected never but got: " + data);
}
function sanitize(html = "") {
  const result = sanitizeHtml(html, {
    allowedTags: ["b", "i", "em", "strong", "a", "p"],
    allowedAttributes: {
      a: ["href", "target"]
    }
  });
  return result;
}
export {
  parseLine,
  renderComponent,
  shouldApplyAnnotation
};
